@@ -12,6 +12,7 @@ use crate::imds::client::token::TokenRuntimePlugin;
 use crate::provider_config::ProviderConfig;
 use crate::PKG_VERSION;
 use aws_runtime::user_agent::{ApiMetadata, AwsUserAgent, UserAgentInterceptor};
+use aws_smithy_runtime::client::metrics::MetricsRuntimePlugin;
 use aws_smithy_runtime::client::orchestrator::operation::Operation;
 use aws_smithy_runtime::client::retries::strategy::StandardRetryStrategy;
 use aws_smithy_runtime_api::box_error::BoxError;
@@ -21,7 +22,7 @@ use aws_smithy_runtime_api::client::endpoint::{
 };
 use aws_smithy_runtime_api::client::interceptors::context::InterceptorContext;
 use aws_smithy_runtime_api::client::orchestrator::{
-    HttpRequest, OrchestratorError, SensitiveOutput,
+    HttpRequest, Metadata, OrchestratorError, SensitiveOutput,
 };
 use aws_smithy_runtime_api::client::result::ConnectorError;
 use aws_smithy_runtime_api::client::result::SdkError;
@@ -476,6 +477,14 @@ impl Builder {
                 common_plugin,
                 self.token_ttl.unwrap_or(DEFAULT_TOKEN_TTL),
             ))
+            .runtime_plugin(
+                MetricsRuntimePlugin::builder()
+                    .with_scope("aws_config::imds_credentials")
+                    .with_time_source(config.time_source())
+                    .with_metadata(Metadata::new("get_credentials", "imds"))
+                    .build()
+                    .expect("All required fields have been set"),
+            )
             .with_connection_poisoning()
             .serializer(|path| {
                 Ok(HttpRequest::try_from(
@@ -638,16 +647,13 @@ pub(crate) mod test {
     use crate::provider_config::ProviderConfig;
     use aws_smithy_async::rt::sleep::TokioSleep;
     use aws_smithy_async::test_util::{instant_time_and_sleep, InstantSleep};
-    use aws_smithy_runtime::client::http::test_util::{
-        capture_request, ReplayEvent, StaticReplayClient,
-    };
+    use aws_smithy_http_client::test_util::{capture_request, ReplayEvent, StaticReplayClient};
     use aws_smithy_runtime::test_util::capture_test_logs::capture_test_logs;
     use aws_smithy_runtime_api::client::interceptors::context::{
         Input, InterceptorContext, Output,
     };
-    use aws_smithy_runtime_api::client::orchestrator::{
-        HttpRequest, HttpResponse, OrchestratorError,
-    };
+    use aws_smithy_runtime_api::client::orchestrator::OrchestratorError;
+    use aws_smithy_runtime_api::client::orchestrator::{HttpRequest, HttpResponse};
     use aws_smithy_runtime_api::client::result::ConnectorError;
     use aws_smithy_runtime_api::client::retries::classifiers::{
         ClassifyRetry, RetryAction, SharedRetryClassifier,
@@ -683,6 +689,7 @@ pub(crate) mod test {
     const TOKEN_A: &str = "AQAEAFTNrA4eEGx0AQgJ1arIq_Cc-t4tWt3fB0Hd8RKhXlKc5ccvhg==";
     const TOKEN_B: &str = "alternatetoken==";
 
+    /// Create a simple token request
     pub(crate) fn token_request(base: &str, ttl: u32) -> HttpRequest {
         http::Request::builder()
             .uri(format!("{}/latest/api/token", base))
@@ -694,6 +701,7 @@ pub(crate) mod test {
             .unwrap()
     }
 
+    /// Create a simple token response
     pub(crate) fn token_response(ttl: u32, token: &'static str) -> HttpResponse {
         HttpResponse::try_from(
             http::Response::builder()
@@ -705,6 +713,7 @@ pub(crate) mod test {
         .unwrap()
     }
 
+    /// Create a simple IMDS request
     pub(crate) fn imds_request(path: &'static str, token: &str) -> HttpRequest {
         http::Request::builder()
             .uri(Uri::from_static(path))
@@ -716,6 +725,7 @@ pub(crate) mod test {
             .unwrap()
     }
 
+    /// Create a simple IMDS response
     pub(crate) fn imds_response(body: &'static str) -> HttpResponse {
         HttpResponse::try_from(
             http::Response::builder()
@@ -726,6 +736,7 @@ pub(crate) mod test {
         .unwrap()
     }
 
+    /// Create an IMDS client with an underlying [StaticReplayClient]
     pub(crate) fn make_imds_client(http_client: &StaticReplayClient) -> super::Client {
         tokio::time::pause();
         super::Client::builder()
@@ -1104,7 +1115,7 @@ pub(crate) mod test {
     #[cfg_attr(windows, ignore)]
     /// Verify that the end-to-end real client has a 1-second connect timeout
     #[tokio::test]
-    #[cfg(feature = "rustls")]
+    #[cfg(feature = "default-https-client")]
     async fn one_second_connect_timeout() {
         use crate::imds::client::ImdsError;
         let client = Client::builder()
