@@ -4,7 +4,7 @@
  */
 
 /* Automatically managed default lints */
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 /* End of automatically managed default lints */
 #![allow(clippy::derive_partial_eq_without_eq)]
 #![warn(
@@ -129,6 +129,8 @@ pub mod ecs;
 mod env_service_config;
 pub mod environment;
 pub mod imds;
+#[cfg(feature = "credentials-login")]
+pub mod login;
 pub mod meta;
 pub mod profile;
 pub mod provider_config;
@@ -223,6 +225,7 @@ mod loader {
     use aws_smithy_async::rt::sleep::{default_async_sleep, AsyncSleep, SharedAsyncSleep};
     use aws_smithy_async::time::{SharedTimeSource, TimeSource};
     use aws_smithy_runtime::client::identity::IdentityCache;
+    use aws_smithy_runtime_api::client::auth::AuthSchemePreference;
     use aws_smithy_runtime_api::client::behavior_version::BehaviorVersion;
     use aws_smithy_runtime_api::client::http::HttpClient;
     use aws_smithy_runtime_api::client::identity::{ResolveCachedIdentity, SharedIdentityCache};
@@ -235,15 +238,17 @@ mod loader {
     use aws_smithy_types::timeout::TimeoutConfig;
     use aws_types::app_name::AppName;
     use aws_types::docs_for;
+    use aws_types::endpoint_config::AccountIdEndpointMode;
     use aws_types::origin::Origin;
     use aws_types::os_shim_internal::{Env, Fs};
     use aws_types::sdk_config::SharedHttpClient;
     use aws_types::SdkConfig;
 
     use crate::default_provider::{
-        app_name, checksums, credentials, disable_request_compression, endpoint_url,
-        ignore_configured_endpoint_urls as ignore_ep, region, request_min_compression_size_bytes,
-        retry_config, timeout_config, use_dual_stack, use_fips,
+        account_id_endpoint_mode, app_name, auth_scheme_preference, checksums, credentials,
+        disable_request_compression, endpoint_url, ignore_configured_endpoint_urls as ignore_ep,
+        region, request_min_compression_size_bytes, retry_config, timeout_config, use_dual_stack,
+        use_fips,
     };
     use crate::meta::region::ProvideRegion;
     #[allow(deprecated)]
@@ -270,9 +275,11 @@ mod loader {
     #[derive(Default, Debug)]
     pub struct ConfigLoader {
         app_name: Option<AppName>,
+        auth_scheme_preference: Option<AuthSchemePreference>,
         identity_cache: Option<SharedIdentityCache>,
         credentials_provider: TriStateOption<SharedCredentialsProvider>,
         token_provider: Option<SharedTokenProvider>,
+        account_id_endpoint_mode: Option<AccountIdEndpointMode>,
         endpoint_url: Option<String>,
         region: Option<Box<dyn ProvideRegion>>,
         retry_config: Option<RetryConfig>,
@@ -396,33 +403,30 @@ mod loader {
         ///
         /// If you wish to use a separate HTTP client for credentials providers when creating clients,
         /// then override the HTTP client set with this function on the client-specific `Config`s.
+        pub fn http_client(mut self, http_client: impl HttpClient + 'static) -> Self {
+            self.http_client = Some(http_client.into_shared());
+            self
+        }
+
+        #[doc = docs_for!(auth_scheme_preference)]
         ///
-        /// ## Examples
-        ///
+        /// # Examples
         /// ```no_run
-        /// # use aws_smithy_async::rt::sleep::SharedAsyncSleep;
-        /// #[cfg(feature = "client-hyper")]
+        /// # use aws_smithy_runtime_api::client::auth::AuthSchemeId;
         /// # async fn create_config() {
-        /// use std::time::Duration;
-        /// use aws_smithy_runtime::client::http::hyper_014::HyperClientBuilder;
-        ///
-        /// let tls_connector = hyper_rustls::HttpsConnectorBuilder::new()
-        ///     .with_webpki_roots()
-        ///     // NOTE: setting `https_only()` will not allow this connector to work with IMDS.
-        ///     .https_only()
-        ///     .enable_http1()
-        ///     .enable_http2()
-        ///     .build();
-        ///
-        /// let hyper_client = HyperClientBuilder::new().build(tls_connector);
-        /// let sdk_config = aws_config::from_env()
-        ///     .http_client(hyper_client)
+        /// let config = aws_config::from_env()
+        ///     // Favors a custom auth scheme over the SigV4 auth scheme.
+        ///     // Note: This will not result in an error, even if the custom scheme is missing from the resolved auth schemes.
+        ///     .auth_scheme_preference([AuthSchemeId::from("custom"), aws_runtime::auth::sigv4::SCHEME_ID])
         ///     .load()
         ///     .await;
         /// # }
         /// ```
-        pub fn http_client(mut self, http_client: impl HttpClient + 'static) -> Self {
-            self.http_client = Some(http_client.into_shared());
+        pub fn auth_scheme_preference(
+            mut self,
+            auth_scheme_preference: impl Into<AuthSchemePreference>,
+        ) -> Self {
+            self.auth_scheme_preference = Some(auth_scheme_preference.into());
             self
         }
 
@@ -651,6 +655,15 @@ mod loader {
             self
         }
 
+        #[doc = docs_for!(account_id_endpoint_mode)]
+        pub fn account_id_endpoint_mode(
+            mut self,
+            account_id_endpoint_mode: AccountIdEndpointMode,
+        ) -> Self {
+            self.account_id_endpoint_mode = Some(account_id_endpoint_mode);
+            self
+        }
+
         /// Override the endpoint URL used for **all** AWS services.
         ///
         /// This method will override the endpoint URL used for **all** AWS services. This primarily
@@ -883,31 +896,13 @@ mod loader {
                 TriStateOption::ExplicitlyUnset => None,
             };
 
-            let token_provider = match self.token_provider {
-                Some(provider) => Some(provider),
-                None => {
-                    #[cfg(feature = "sso")]
-                    {
-                        let mut builder =
-                            crate::default_provider::token::DefaultTokenChain::builder()
-                                .configure(conf.clone());
-                        builder.set_region(region.clone());
-                        Some(SharedTokenProvider::new(builder.build().await))
-                    }
-                    #[cfg(not(feature = "sso"))]
-                    {
-                        None
-                    }
-                }
-            };
-
             let profiles = conf.profile().await;
             let service_config = EnvServiceConfig {
                 env: conf.env(),
                 env_config_sections: profiles.cloned().unwrap_or_default(),
             };
             let mut builder = SdkConfig::builder()
-                .region(region)
+                .region(region.clone())
                 .retry_config(retry_config)
                 .timeout_config(timeout_config)
                 .time_source(time_source)
@@ -939,6 +934,30 @@ mod loader {
                 }
             };
 
+            let token_provider = match self.token_provider {
+                Some(provider) => {
+                    builder.insert_origin("token_provider", Origin::shared_config());
+                    Some(provider)
+                }
+                None => {
+                    #[cfg(feature = "sso")]
+                    {
+                        let mut builder =
+                            crate::default_provider::token::DefaultTokenChain::builder()
+                                .configure(conf.clone());
+                        builder.set_region(region);
+                        Some(SharedTokenProvider::new(builder.build().await))
+                    }
+                    #[cfg(not(feature = "sso"))]
+                    {
+                        None
+                    }
+                    // Not setting `Origin` in this arm, and that's good for now as long as we know
+                    // it's not programmatically set in the shared config.
+                    // We can consider adding `Origin::Default` if needed.
+                }
+            };
+
             builder.set_endpoint_url(endpoint_url);
             builder.set_behavior_version(self.behavior_version);
             builder.set_http_client(self.http_client);
@@ -946,6 +965,7 @@ mod loader {
 
             let identity_cache = match self.identity_cache {
                 None => match self.behavior_version {
+                    #[allow(deprecated)]
                     Some(bv) if bv.is_at_least(BehaviorVersion::v2024_03_28()) => {
                         Some(IdentityCache::lazy().build())
                     }
@@ -968,6 +988,23 @@ mod loader {
                     checksums::response_checksum_validation_provider(&conf).await
                 };
 
+            let account_id_endpoint_mode =
+                if let Some(acccount_id_endpoint_mode) = self.account_id_endpoint_mode {
+                    Some(acccount_id_endpoint_mode)
+                } else {
+                    account_id_endpoint_mode::account_id_endpoint_mode_provider(&conf).await
+                };
+
+            let auth_scheme_preference =
+                if let Some(auth_scheme_preference) = self.auth_scheme_preference {
+                    builder.insert_origin("auth_scheme_preference", Origin::shared_config());
+                    Some(auth_scheme_preference)
+                } else {
+                    auth_scheme_preference::auth_scheme_preference_provider(&conf).await
+                    // Not setting `Origin` in this arm, and that's good for now as long as we know
+                    // it's not programmatically set in the shared config.
+                };
+
             builder.set_request_checksum_calculation(request_checksum_calculation);
             builder.set_response_checksum_validation(response_checksum_validation);
             builder.set_identity_cache(identity_cache);
@@ -979,6 +1016,8 @@ mod loader {
             builder.set_disable_request_compression(disable_request_compression);
             builder.set_request_min_compression_size_bytes(request_min_compression_size_bytes);
             builder.set_stalled_stream_protection(self.stalled_stream_protection_config);
+            builder.set_account_id_endpoint_mode(account_id_endpoint_mode);
+            builder.set_auth_scheme_preference(auth_scheme_preference);
             builder.build()
         }
     }
@@ -1005,7 +1044,7 @@ mod loader {
         use crate::{defaults, ConfigLoader};
         use aws_credential_types::provider::ProvideCredentials;
         use aws_smithy_async::rt::sleep::TokioSleep;
-        use aws_smithy_runtime::client::http::test_util::{infallible_client_fn, NeverClient};
+        use aws_smithy_http_client::test_util::{infallible_client_fn, NeverClient};
         use aws_smithy_runtime::test_util::capture_test_logs::capture_test_logs;
         use aws_types::app_name::AppName;
         use aws_types::origin::Origin;
@@ -1215,7 +1254,7 @@ mod loader {
             );
         }
 
-        #[cfg(feature = "rustls")]
+        #[cfg(feature = "default-https-client")]
         #[tokio::test]
         async fn disable_default_credentials() {
             let config = defaults(BehaviorVersion::latest())
@@ -1225,7 +1264,7 @@ mod loader {
             assert!(config.credentials_provider().is_none());
         }
 
-        #[cfg(feature = "rustls")]
+        #[cfg(feature = "default-https-client")]
         #[tokio::test]
         async fn identity_cache_defaulted() {
             let config = defaults(BehaviorVersion::latest()).load().await;
@@ -1233,7 +1272,7 @@ mod loader {
             assert!(config.identity_cache().is_some());
         }
 
-        #[cfg(feature = "rustls")]
+        #[cfg(feature = "default-https-client")]
         #[allow(deprecated)]
         #[tokio::test]
         async fn identity_cache_old_behavior_version() {
