@@ -449,8 +449,14 @@ impl<'a> SettersCtx<'a> {
         let pats = imp.inputs.iter().map(|(pat, _)| pat);
         let types = imp.inputs.iter().map(|(_, ty)| ty);
         let const_ = &self.base.const_;
+        let fn_modifiers = self.member.respan(quote!(#vis #const_));
 
-        quote! {
+        // It's important to keep the span of `self` the same across all
+        // references to it. Otherwise `self`s that have different spans will
+        // be treated as totally different symbols due to the hygiene rules.
+        let self_ = quote!(self);
+
+        quote_spanned! {self.member.span=>
             #( #docs )*
             #[allow(
                 // This is intentional. We want the builder syntax to compile away
@@ -461,9 +467,13 @@ impl<'a> SettersCtx<'a> {
                 // your design of this setter already went wrong.
                 clippy::impl_trait_in_params,
                 clippy::missing_const_for_fn,
+                // When having a field which has one of the prefixes listed by
+                // `clippy::wrong_self_convention` you will end up getting said lint
+                // warning in your `bon::Builder` because we take self by value.
+                clippy::wrong_self_convention,
             )]
             #[inline(always)]
-            #vis #const_ fn #name(#maybe_mut self, #( #pats: #types ),*) -> #return_type
+            #(#fn_modifiers)* fn #name(#maybe_mut #self_, #( #pats: #types ),*) -> #return_type
             #where_clause
             {
                 #body
