@@ -3,7 +3,7 @@ use std::{borrow::Cow, collections::HashSet, ops::Not, str::FromStr, sync::Arc};
 use nom::{
     branch::alt,
     bytes::complete::{tag, take_till1, take_until, take_while, take_while1},
-    character::complete::{char, multispace0, one_of, space0},
+    character::complete::{char, multispace0, multispace1, one_of, space0},
     combinator::{opt, recognize},
     error::{context, ContextError, ParseError},
     multi::{separated_list0, separated_list1},
@@ -20,8 +20,10 @@ use super::{
     matcher::{StringMatcher, StringMatcherParseError},
     MatchSpec,
 };
+use crate::match_spec::condition::parse_condition;
 use crate::{
     build_spec::{BuildNumberSpec, ParseBuildNumberSpecError},
+    match_spec::package_name_matcher::{PackageNameMatcher, PackageNameMatcherParseError},
     package::ArchiveIdentifier,
     utils::{path::is_absolute_path, url::parse_scheme},
     version_spec::{
@@ -29,10 +31,8 @@ use crate::{
         version_tree::{recognize_constraint, recognize_version},
         ParseVersionSpecError,
     },
-    Channel, ChannelConfig, InvalidPackageNameError, NamelessMatchSpec, PackageName,
-    ParseChannelError, ParseStrictness,
-    ParseStrictness::{Lenient, Strict},
-    ParseVersionError, Platform, VersionSpec,
+    Channel, ChannelConfig, NamelessMatchSpec, ParseChannelError, ParseMatchSpecOptions,
+    ParseStrictness, ParseVersionError, Platform, VersionSpec,
 };
 
 /// The type of parse error that occurred when parsing match spec.
@@ -96,20 +96,36 @@ pub enum ParseMatchSpecError {
     #[error("unable to parse hash digest from hex")]
     InvalidHashDigest,
 
-    /// The package name was invalid
+    /// The package name matcher was invalid
     #[error(transparent)]
-    InvalidPackageName(#[from] InvalidPackageNameError),
+    InvalidPackageNameMatcher(#[from] PackageNameMatcherParseError),
 
     /// Multiple values for a key in the matchspec
     #[error("found multiple values for: {0}")]
     MultipleValueForKey(String),
+
+    /// More than one semicolon in match spec
+    #[error("more than one semicolon in match spec")]
+    MoreThanOneSemicolon,
+
+    /// Invalid condition in match spec
+    #[error("could not parse condition {0}: {1}")]
+    InvalidCondition(String, String),
+
+    /// Only exact package name matchers are allowed but a glob was provided
+    #[error("\"{0}\" looks like a glob but only exact package names are allowed, package names can only contain 0-9, a-z, A-Z, -, _, or .")]
+    OnlyExactPackageNameMatchersAllowedGlob(String),
+
+    /// Only exact package name matchers are allowed but a regex was provided
+    #[error("\"{0}\" looks like a regex but only exact package names are allowed, package names can only contain 0-9, a-z, A-Z, -, _, or .")]
+    OnlyExactPackageNameMatchersAllowedRegex(String),
 }
 
 impl FromStr for MatchSpec {
     type Err = ParseMatchSpecError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Self::from_str(s, Lenient)
+        Self::from_str(s, ParseMatchSpecOptions::default())
     }
 }
 
@@ -117,9 +133,9 @@ impl MatchSpec {
     /// Parses a [`MatchSpec`] from a string with a given strictness.
     pub fn from_str(
         source: &str,
-        strictness: ParseStrictness,
+        options: impl Into<ParseMatchSpecOptions>,
     ) -> Result<Self, ParseMatchSpecError> {
-        matchspec_parser(source, strictness)
+        matchspec_parser(source, options.into())
     }
 }
 
@@ -134,12 +150,39 @@ fn strip_comment(input: &str) -> (&str, Option<&str>) {
 
 /// Strips any if statements from the matchspec. `if` statements in matchspec
 /// are "anticipating future compatibility issues".
-fn strip_if(input: &str) -> (&str, Option<&str>) {
-    // input
-    //     .split_once("if")
-    //     .map(|(spec, if_statement)| (spec, Some(if_statement)))
-    //     .unwrap_or_else(|| (input, None))
-    (input, None)
+fn strip_if(input: &str) -> Result<(&str, Option<&str>), ParseMatchSpecError> {
+    // Check that we only have a single `if` statement (semicolon separated)
+    if input.matches(';').count() > 1 {
+        return Err(ParseMatchSpecError::MoreThanOneSemicolon);
+    }
+
+    // Try to parse with nom for better whitespace handling
+    if let Ok((matchspec_str, condition)) = parse_if_statement(input) {
+        Ok((matchspec_str.trim(), Some(condition.trim())))
+    } else {
+        // No condition found, return the input as is
+        Ok((input.trim(), None))
+    }
+}
+
+/// Parse the if statement structure with flexible whitespace
+fn parse_if_statement(input: &str) -> IResult<&str, &str> {
+    let (remaining, (matchspec_part, _)) = (
+        // Take everything up to "; if"
+        nom::bytes::complete::take_until(";"),
+        // Match "; if " with flexible whitespace
+        (
+            multispace0,
+            char(';'),
+            multispace0,
+            tag("if"),
+            multispace1, // At least one whitespace after "if"
+        ),
+    )
+        .parse(input)?;
+
+    // Return the condition part and the matchspec part
+    Ok((matchspec_part, remaining))
 }
 
 /// An optimized data structure to store key value pairs in between a bracket
@@ -229,7 +272,6 @@ fn strip_brackets(input: &str) -> Result<(Cow<'_, str>, BracketVec<'_>), ParseMa
     }
 }
 
-#[cfg(feature = "experimental_extras")]
 /// Parses a list of optional dependencies from a string `feat1, feat2, feat3]`
 /// -> `vec![feat1, feat2, feat3]`.
 pub fn parse_extras(input: &str) -> Result<Vec<String>, ParseMatchSpecError> {
@@ -261,11 +303,11 @@ pub fn parse_extras(input: &str) -> Result<Vec<String>, ParseMatchSpecError> {
 fn parse_bracket_vec_into_components(
     bracket: BracketVec<'_>,
     match_spec: NamelessMatchSpec,
-    strictness: ParseStrictness,
+    options: ParseMatchSpecOptions,
 ) -> Result<NamelessMatchSpec, ParseMatchSpecError> {
     let mut match_spec = match_spec;
 
-    if strictness == Strict {
+    if options.strictness() == ParseStrictness::Strict {
         // check for duplicate keys
         let mut seen = HashSet::new();
         for (key, _) in &bracket {
@@ -279,17 +321,16 @@ fn parse_bracket_vec_into_components(
     for elem in bracket {
         let (key, value) = elem;
         match key {
-            "version" => match_spec.version = Some(VersionSpec::from_str(value, strictness)?),
+            "version" => {
+                match_spec.version = Some(VersionSpec::from_str(value, options.strictness())?);
+            }
             "build" => match_spec.build = Some(StringMatcher::from_str(value)?),
             "build_number" => match_spec.build_number = Some(BuildNumberSpec::from_str(value)?),
             "extras" => {
                 // Optional features are still experimental
-                #[cfg(feature = "experimental_extras")]
-                {
+                if options.allow_experimental_extras() {
                     match_spec.extras = Some(parse_extras(value)?);
-                }
-                #[cfg(not(feature = "experimental_extras"))]
-                {
+                } else {
                     return Err(ParseMatchSpecError::InvalidBracketKey("extras".to_string()));
                 }
             }
@@ -362,7 +403,10 @@ pub fn parse_url_like(input: &str) -> Result<Option<Url>, ParseMatchSpecError> {
 }
 
 /// Strip the package name from the input.
-fn strip_package_name(input: &str) -> Result<(Option<PackageName>, &str), ParseMatchSpecError> {
+fn strip_package_name(
+    input: &str,
+    exact_names_only: bool,
+) -> Result<(Option<PackageNameMatcher>, &str), ParseMatchSpecError> {
     let (rest, package_name) =
         take_while1(|c: char| !c.is_whitespace() && !is_start_of_version_constraint(c))(
             input.trim(),
@@ -375,15 +419,42 @@ fn strip_package_name(input: &str) -> Result<(Option<PackageName>, &str), ParseM
         return Err(ParseMatchSpecError::MissingPackageName);
     }
 
+    let rest = rest.trim();
+
     // Handle asterisk as a wildcard (no package name)
     if trimmed_package_name == "*" {
-        return Ok((None, rest.trim()));
+        return Ok((None, rest));
     }
 
-    Ok((
-        Some(PackageName::from_str(trimmed_package_name)?),
-        rest.trim(),
-    ))
+    let package_name = match PackageNameMatcher::from_str(trimmed_package_name)
+        .map_err(ParseMatchSpecError::InvalidPackageNameMatcher)?
+    {
+        PackageNameMatcher::Exact(name) => PackageNameMatcher::Exact(name),
+        PackageNameMatcher::Glob(glob) => {
+            if exact_names_only {
+                return Err(
+                    ParseMatchSpecError::OnlyExactPackageNameMatchersAllowedGlob(
+                        glob.as_str().to_string(),
+                    ),
+                );
+            } else {
+                PackageNameMatcher::Glob(glob)
+            }
+        }
+        PackageNameMatcher::Regex(regex) => {
+            if exact_names_only {
+                return Err(
+                    ParseMatchSpecError::OnlyExactPackageNameMatchersAllowedRegex(
+                        regex.as_str().to_string(),
+                    ),
+                );
+            } else {
+                PackageNameMatcher::Regex(regex)
+            }
+        }
+    };
+
+    Ok((Some(package_name), rest))
 }
 
 /// Splits a string into version and build constraints.
@@ -395,7 +466,7 @@ fn split_version_and_build(
         strictness: ParseStrictness,
     ) -> impl FnMut(&'a str) -> IResult<&'a str, &'a str, E> {
         move |input: &'a str| {
-            if strictness == Lenient {
+            if strictness == ParseStrictness::Lenient {
                 alt((parse_special_equality, recognize_constraint)).parse(input)
             } else {
                 recognize_constraint(input)
@@ -451,7 +522,7 @@ fn split_version_and_build(
         strictness: ParseStrictness,
     ) -> impl FnMut(&'a str) -> IResult<&'a str, &'a str, E> {
         move |input: &'a str| {
-            if strictness == Lenient {
+            if strictness == ParseStrictness::Lenient {
                 terminated(parse_version_group(strictness), opt(one_of(" ="))).parse(input)
             } else {
                 terminated(parse_version_group(strictness), space0).parse(input)
@@ -464,7 +535,7 @@ fn split_version_and_build(
             let build_string = rest.trim();
 
             // Check validity of the build string
-            if strictness == Strict
+            if strictness == ParseStrictness::Strict
                 && build_string.contains(|c: char| !c.is_alphanumeric() && c != '_' && c != '*')
             {
                 return Err(ParseMatchSpecError::InvalidBuildString(
@@ -522,13 +593,17 @@ impl FromStr for NamelessMatchSpec {
     type Err = ParseMatchSpecError;
 
     fn from_str(input: &str) -> Result<Self, Self::Err> {
-        Self::from_str(input, Lenient)
+        Self::from_str(input, ParseMatchSpecOptions::default())
     }
 }
 
 impl NamelessMatchSpec {
     /// Parses a [`NamelessMatchSpec`] from a string with a given strictness.
-    pub fn from_str(input: &str, strictness: ParseStrictness) -> Result<Self, ParseMatchSpecError> {
+    pub fn from_str(
+        input: &str,
+        options: impl Into<ParseMatchSpecOptions>,
+    ) -> Result<Self, ParseMatchSpecError> {
+        let options = options.into();
         // Strip off brackets portion
         let (input, brackets) = strip_brackets(input.trim())?;
         let input = input.trim();
@@ -542,7 +617,7 @@ impl NamelessMatchSpec {
         }
 
         let mut match_spec =
-            parse_bracket_vec_into_components(brackets, NamelessMatchSpec::default(), strictness)?;
+            parse_bracket_vec_into_components(brackets, NamelessMatchSpec::default(), options)?;
 
         // 5. Strip of ':' to find channel and namespace
         // This assumes the [*] portions is stripped off, and then strip reverse to
@@ -566,8 +641,8 @@ impl NamelessMatchSpec {
 
         // Get the version and optional build string
         if !input.is_empty() {
-            let (version, build) = parse_version_and_build(input, strictness)?;
-            if strictness == Strict {
+            let (version, build) = parse_version_and_build(input, options.strictness())?;
+            if options.strictness() == ParseStrictness::Strict {
                 if match_spec.version.is_some() && version.is_some() {
                     return Err(ParseMatchSpecError::MultipleValueForKey(
                         "version".to_owned(),
@@ -608,18 +683,24 @@ fn parse_channel_and_subdir(
 
 /// Parses a conda match spec.
 /// This is based on: <https://github.com/conda/conda/blob/master/conda/models/match_spec.py#L569>
-fn matchspec_parser(
+pub(crate) fn matchspec_parser(
     input: &str,
-    strictness: ParseStrictness,
+    options: ParseMatchSpecOptions,
 ) -> Result<MatchSpec, ParseMatchSpecError> {
     // Step 1. Strip '#' and `if` statement
     let (input, _comment) = strip_comment(input);
-    let (input, _if_clause) = strip_if(input);
+
+    let (input, condition) = if options.allow_experimental_conditionals() {
+        strip_if(input)?
+    } else {
+        let (input, _condition) = strip_if(input)?;
+        (input, None)
+    };
 
     // 2. Strip off brackets portion
     let (input, brackets) = strip_brackets(input.trim())?;
     let mut nameless_match_spec =
-        parse_bracket_vec_into_components(brackets, NamelessMatchSpec::default(), strictness)?;
+        parse_bracket_vec_into_components(brackets, NamelessMatchSpec::default(), options)?;
 
     // 3. Strip off parens portion
     // TODO: What is this? I've never seen it
@@ -628,7 +709,7 @@ fn matchspec_parser(
     if nameless_match_spec.url.is_none() {
         if let Some(url) = parse_url_like(&input)? {
             let archive = ArchiveIdentifier::try_from_url(&url);
-            let name = archive.and_then(|a| a.try_into().ok());
+            let name = archive.and_then(|a| PackageNameMatcher::from_str(&a.name).ok());
 
             // TODO: This should also work without a proper name from the url filename
             if name.is_none() {
@@ -667,14 +748,14 @@ fn matchspec_parser(
     }
 
     // Step 6. Strip off the package name from the input
-    let (name, input) = strip_package_name(input)?;
+    let (name, input) = strip_package_name(input, options.exact_names_only())?;
     let mut match_spec = MatchSpec::from_nameless(nameless_match_spec, name);
 
     // Step 7. Otherwise, sort our version + build
     let input = input.trim();
     if !input.is_empty() {
-        let (version, build) = parse_version_and_build(input, strictness)?;
-        if strictness == Strict {
+        let (version, build) = parse_version_and_build(input, options.strictness())?;
+        if options.strictness() == ParseStrictness::Strict {
             if match_spec.version.is_some() && version.is_some() {
                 return Err(ParseMatchSpecError::MultipleValueForKey(
                     "version".to_owned(),
@@ -689,6 +770,21 @@ fn matchspec_parser(
         match_spec.build = match_spec.build.or(build);
     }
 
+    if let Some(condition) = condition {
+        let (remainder, condition) = parse_condition(condition).map_err(|e| {
+            ParseMatchSpecError::InvalidCondition(condition.to_string(), e.to_string())
+        })?;
+
+        if remainder.trim().is_empty().not() {
+            return Err(ParseMatchSpecError::InvalidCondition(
+                condition.to_string(),
+                "remainder not empty".to_string(),
+            ));
+        }
+
+        match_spec.condition = Some(condition);
+    }
+
     Ok(match_spec)
 }
 
@@ -712,7 +808,7 @@ fn optionally_strip_equals<'a>(
 
     // If we are not in lenient mode then stop processing at this point. Any other
     // special case parsing behavior is only part of lenient mode parsing.
-    if strictness != Lenient {
+    if strictness != ParseStrictness::Lenient {
         return version_str.into();
     }
 
@@ -764,12 +860,11 @@ mod tests {
         parse_channel_and_subdir, split_version_and_build, strip_brackets, strip_package_name,
         BracketVec, MatchSpec, ParseMatchSpecError,
     };
-    #[cfg(feature = "experimental_extras")]
     use crate::match_spec::parse::parse_extras;
     use crate::{
         match_spec::parse::parse_bracket_list, BuildNumberSpec, Channel, ChannelConfig,
-        NamelessMatchSpec, ParseChannelError, ParseStrictness, ParseStrictness::*, Version,
-        VersionSpec,
+        NamelessMatchSpec, ParseChannelError, ParseMatchSpecOptions, ParseStrictness,
+        ParseStrictness::*, Version, VersionSpec,
     };
 
     fn channel_config() -> ChannelConfig {
@@ -1272,8 +1367,10 @@ mod tests {
 
     #[test]
     fn test_missing_package_name() {
-        let package_name = strip_package_name("");
-        assert_matches!(package_name, Err(ParseMatchSpecError::MissingPackageName));
+        for exact_names_only in [true, false] {
+            let package_name = strip_package_name("", exact_names_only);
+            assert_matches!(package_name, Err(ParseMatchSpecError::MissingPackageName));
+        }
     }
 
     #[test]
@@ -1282,6 +1379,15 @@ mod tests {
         assert!(spec.namespace.is_none());
     }
 
+    #[test]
+    fn test_multiple_semicolons() {
+        let spec = MatchSpec::from_str("foo; if bar; if baz", Strict);
+        assert_matches!(spec, Err(ParseMatchSpecError::MoreThanOneSemicolon));
+
+        let spec2 = MatchSpec::from_str("package; something; else", Lenient);
+        assert_matches!(spec2, Err(ParseMatchSpecError::MoreThanOneSemicolon));
+    }
+
     #[test]
     fn test_namespace() {
         // Test with url channel and url in brackets
@@ -1352,7 +1458,7 @@ mod tests {
             .expect_err("Should try to parse as name not url");
         assert_eq!(
             err.to_string(),
-            "'bla/bla' is not a valid package name. Package names can only contain 0-9, a-z, A-Z, -, _, or ."
+            "invalid package name 'bla/bla': 'bla/bla' is not a valid package name. Package names can only contain 0-9, a-z, A-Z, -, _, or ."
         );
     }
 
@@ -1368,7 +1474,7 @@ mod tests {
     fn test_issue_717() {
         assert_matches!(
             MatchSpec::from_str("ray[default,data] >=2.9.0,<3.0.0", Strict),
-            Err(ParseMatchSpecError::InvalidPackageName(_))
+            Err(ParseMatchSpecError::InvalidPackageNameMatcher(_))
         );
     }
 
@@ -1450,6 +1556,7 @@ mod tests {
                 .unwrap(),
             ),
             license: Some("MIT".into()),
+            condition: None,
         });
 
         // insta check all the strings
@@ -1476,26 +1583,56 @@ mod tests {
         assert!(version_spec.matches(&version));
     }
 
-    #[cfg(feature = "experimental_extras")]
     #[test]
+    fn test_conditional_parsing() {
+        let spec = MatchSpec::from_str(
+            "foo; if python >=3.6",
+            ParseMatchSpecOptions::strict().with_experimental_conditionals(true),
+        )
+        .unwrap();
+        assert_eq!(spec.name, Some("foo".parse().unwrap()));
+        assert_eq!(
+            spec.condition.unwrap().to_string(),
+            "python >=3.6".to_string()
+        );
+    }
+
+    #[test]
+    fn test_conditional_parsing_disabled() {
+        let spec = MatchSpec::from_str("foo; if python >=3.6", Strict).unwrap();
+        assert_eq!(spec.name, Some("foo".parse().unwrap()));
+        assert!(spec.condition.is_none());
+    }
+
+    #[test]
     fn test_simple_extras() {
-        let spec = MatchSpec::from_str("foo[extras=[bar]]", Strict).unwrap();
+        let spec = MatchSpec::from_str(
+            "foo[extras=[bar]]",
+            ParseMatchSpecOptions::strict().with_experimental_extras(true),
+        )
+        .unwrap();
 
         assert_eq!(spec.extras, Some(vec!["bar".to_string()]));
-        assert!(MatchSpec::from_str("foo[extras=[bar,baz]", Strict).is_err());
+        assert!(MatchSpec::from_str(
+            "foo[extras=[bar,baz]",
+            ParseMatchSpecOptions::strict().with_experimental_extras(true)
+        )
+        .is_err());
     }
 
-    #[cfg(feature = "experimental_extras")]
     #[test]
     fn test_multiple_extras() {
-        let spec = MatchSpec::from_str("foo[extras=[bar,baz]]", Strict).unwrap();
+        let spec = MatchSpec::from_str(
+            "foo[extras=[bar,baz]]",
+            ParseMatchSpecOptions::strict().with_experimental_extras(true),
+        )
+        .unwrap();
         assert_eq!(
             spec.extras,
             Some(vec!["bar".to_string(), "baz".to_string()])
         );
     }
 
-    #[cfg(feature = "experimental_extras")]
     #[test]
     fn test_parse_extras() {
         assert_eq!(
@@ -1510,35 +1647,50 @@ mod tests {
         assert!(parse_extras("[bar,baz]").is_err());
     }
 
-    #[cfg(feature = "experimental_extras")]
     #[test]
     fn test_invalid_extras() {
+        let opts = ParseMatchSpecOptions::strict().with_experimental_extras(true);
+
         // Empty extras value
-        assert!(MatchSpec::from_str("foo[extras=]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=]", opts).is_err());
 
         // Missing brackets around extras list
-        assert!(MatchSpec::from_str("foo[extras=bar,baz]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=bar,baz]", opts).is_err());
 
         // Trailing comma in extras list
-        assert!(MatchSpec::from_str("foo[extras=[bar,]]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=[bar,]]", opts).is_err());
 
         // Invalid characters in extras name
-        assert!(MatchSpec::from_str("foo[extras=[bar!,baz]]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=[bar!,baz]]", opts).is_err());
 
         // Invalid characters in extras name
-        println!(
-            "{:?}",
-            MatchSpec::from_str("foo[extras=[bar!,baz]]", Strict)
-        );
-        assert!(MatchSpec::from_str("foo[extras=[bar!,baz]]", Strict).is_err());
+        println!("{:?}", MatchSpec::from_str("foo[extras=[bar!,baz]]", opts));
+        assert!(MatchSpec::from_str("foo[extras=[bar!,baz]]", opts).is_err());
 
         // Empty extras item
-        assert!(MatchSpec::from_str("foo[extras=[bar,,baz]]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=[bar,,baz]]", opts).is_err());
 
         // Missing closing bracket
-        assert!(MatchSpec::from_str("foo[extras=[bar,baz", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=[bar,baz", opts).is_err());
 
         // Missing opening bracket
-        assert!(MatchSpec::from_str("foo[extras=bar,baz]]", Strict).is_err());
+        assert!(MatchSpec::from_str("foo[extras=bar,baz]]", opts).is_err());
+    }
+
+    #[test]
+    fn test_glob_and_regex_error_messages() {
+        // Test glob error message
+        let glob_err = MatchSpec::from_str("bla*", Strict).unwrap_err();
+        assert_eq!(
+            glob_err.to_string(),
+            "\"bla*\" looks like a glob but only exact package names are allowed, package names can only contain 0-9, a-z, A-Z, -, _, or ."
+        );
+
+        // Test regex error message
+        let regex_err = MatchSpec::from_str("^foo.*$", Strict).unwrap_err();
+        assert_eq!(
+            regex_err.to_string(),
+            "\"^foo.*$\" looks like a regex but only exact package names are allowed, package names can only contain 0-9, a-z, A-Z, -, _, or ."
+        );
     }
 }
