@@ -5,7 +5,6 @@ use std::{
     sync::Arc,
 };
 
-use fxhash::FxHashMap;
 use indexmap::IndexSet;
 use itertools::Either;
 use pep508_rs::ExtraName;
@@ -176,7 +175,7 @@ fn parse_from_lock<P>(
     }
 
     // Determine the indices of the packages by url
-    let mut conda_url_lookup: FxHashMap<UrlOrPath, Vec<_>> = FxHashMap::default();
+    let mut conda_url_lookup: ahash::HashMap<UrlOrPath, Vec<_>> = ahash::HashMap::default();
     for (idx, conda_package) in conda_packages.iter().enumerate() {
         conda_url_lookup
             .entry(conda_package.location().clone())
@@ -188,7 +187,7 @@ fn parse_from_lock<P>(
         .iter()
         .enumerate()
         .map(|(idx, p)| (&p.location, idx))
-        .collect::<FxHashMap<_, _>>();
+        .collect::<ahash::HashMap<_, _>>();
     let mut pypi_runtime_lookup = IndexSet::new();
 
     let environments = raw
