@@ -19,6 +19,15 @@ pub enum Authentication {
     },
     /// A conda token is sent in the URL as `/t/{TOKEN}/...`
     CondaToken(String),
+    /// S3 credentials
+    S3Credentials {
+        /// The access key ID to use for S3 authentication
+        access_key_id: String,
+        /// The secret access key to use for S3 authentication
+        secret_access_key: String,
+        /// The session token to use for S3 authentication
+        session_token: Option<String>,
+    },
 }
 
 /// An error that can occur when parsing an authentication string
@@ -38,3 +47,15 @@ impl FromStr for Authentication {
         serde_json::from_str(s).map_err(|_err| AuthenticationParseError::InvalidToken)
     }
 }
+
+impl Authentication {
+    /// Get the scheme of the authentication method
+    pub fn method(&self) -> &str {
+        match self {
+            Authentication::BearerToken(_) => "BearerToken",
+            Authentication::BasicHTTP { .. } => "BasicHTTP",
+            Authentication::CondaToken(_) => "CondaToken",
+            Authentication::S3Credentials { .. } => "S3",
+        }
+    }
+}
