@@ -46,8 +46,8 @@ pub enum GatewayError {
         #[source] super::direct_url_query::DirectUrlQueryError,
     ),
 
-    #[error("the match spec '{0}' does not specify a name")]
-    MatchSpecWithoutName(Box<MatchSpec>),
+    #[error("the match spec '{0}' does not specify an exact name")]
+    MatchSpecWithoutExactName(Box<MatchSpec>),
 
     #[error("the package from url '{0}', doesn't have the same name as the match spec filename intents '{1}'")]
     UrlRecordNameMismatch(String, String),
