@@ -488,6 +488,11 @@ macro_rules! solver_backend_tests {
             crate::min_age_tests::solve_min_age_exempt_no_timestamp::<$T>();
         }
 
+        #[test]
+        fn resolvo_issue_188() {
+            crate::solver_case_tests::resolvo_issue_188::<$T>();
+        }
+
         #[test]
         fn test_duplicate_record() {
             use rattler_solve::SolverImpl;
