@@ -25,9 +25,13 @@ mod real;
 #[cfg(all(feature = "wasm", target_arch = "wasm32"))]
 mod wasm;
 
+#[cfg(feature = "memory")]
+pub use in_memory::InMemoryDirEntry;
 #[cfg(feature = "memory")]
 pub use in_memory::InMemoryFile;
 #[cfg(feature = "memory")]
+pub use in_memory::InMemoryMetadata;
+#[cfg(feature = "memory")]
 pub use in_memory::InMemorySys;
 
 #[cfg(all(feature = "wasm", target_arch = "wasm32"))]
